@@ -2,7 +2,7 @@
 /**
  * Defines the AJAX actions used in Kronolith.
  *
- * Copyright 2012-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2012-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -32,13 +32,14 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
     public function listCalendars()
     {
         Kronolith::initialize();
+        $all_external_calendars = $GLOBALS['calendar_manager']->get(Kronolith::ALL_EXTERNAL_CALENDARS);
         $result = new stdClass;
         $auth_name = $GLOBALS['registry']->getAuth();
 
         // Calendars. Do some twisting to sort own calendar before shared
         // calendars.
         foreach (array(true, false) as $my) {
-            foreach ($GLOBALS['all_calendars'] as $id => $calendar) {
+            foreach ($GLOBALS['calendar_manager']->get(Kronolith::ALL_CALENDARS) as $id => $calendar) {
                 $owner = ($auth_name && ($calendar->owner() == $auth_name));
                 if (($my && $owner) || (!$my && !$owner)) {
                     $result->calendars['internal'][$id] = $calendar->toHash();
@@ -48,12 +49,12 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
             // Tasklists
             if (Kronolith::hasApiPermission('tasks')) {
                 foreach ($GLOBALS['registry']->tasks->listTasklists($my, Horde_Perms::SHOW, false) as $id => $tasklist) {
-                    if (isset($GLOBALS['all_external_calendars']['tasks/' . $id])) {
+                    if (isset($all_external_calendars['tasks/' . $id])) {
                         $owner = ($auth_name &&
                                   ($tasklist->get('owner') == $auth_name));
                         if (($my && $owner) || (!$my && !$owner)) {
                             $result->calendars['tasklists']['tasks/' . $id] =
-                                $GLOBALS['all_external_calendars']['tasks/' . $id]->toHash();
+                                $all_external_calendars['tasks/' . $id]->toHash();
                         }
                     }
                 }
@@ -78,19 +79,19 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
         }
 
         // Timeobjects
-        foreach ($GLOBALS['all_external_calendars'] as $id => $calendar) {
+        foreach ($all_external_calendars as $id => $calendar) {
             if ($calendar->api() != 'tasks' && $calendar->display()) {
                 $result->calendars['external'][$id] = $calendar->toHash();
             }
         }
 
         // Remote calendars
-        foreach ($GLOBALS['all_remote_calendars'] as $url => $calendar) {
+        foreach ($GLOBALS['calendar_manager']->get(Kronolith::ALL_REMOTE_CALENDARS) as $url => $calendar) {
             $result->calendars['remote'][$url] = $calendar->toHash();
         }
 
         // Holidays
-        foreach ($GLOBALS['all_holidays'] as $id => $calendar) {
+        foreach ($GLOBALS['calendar_manager']->get(Kronolith::ALL_HOLIDAYS) as $id => $calendar) {
             $result->calendars['holiday'][$id] = $calendar->toHash();
         }
 
@@ -102,6 +103,8 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
      */
     public function listEvents()
     {
+        global $session;
+
         $start = new Horde_Date($this->vars->start);
         $end   = new Horde_Date($this->vars->end);
         $result = $this->_signedResponse($this->vars->cal);
@@ -109,12 +112,12 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
             return $result;
         }
         try {
-            session_write_close();
+            $session->close();
             $events = $kronolith_driver->listEvents($start, $end, array(
                 'show_recurrence' => true,
                 'json' => true)
             );
-            session_start();
+            $session->start();
             if (count($events)) {
                 $result->events = $events;
             }
@@ -125,17 +128,16 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
     }
 
     /**
-     * Return a JSON object representing the requested event.
+     * Returns a JSON object representing the requested event.
+     *
      * Request variables used:
-     *<pre>
-     *  -cal    The calendar id
-     *  -id     The event id
-     *  -date   The date of the event we are requesting [OPTIONAL]
-     *  -rsd    The event start date of the instance of a recurring event, if
+     *  - cal:  The calendar id
+     *  - id:   The event id
+     *  - date: The date of the event we are requesting [OPTIONAL]
+     *  - rsd:  The event start date of the instance of a recurring event, if
      *          requesting a specific instance.
-     *  -red    The event end date of the instance of a recurring event, if
+     *  - red:  The event end date of the instance of a recurring event, if
      *          requesting a specific instance.
-     *</pre>
      */
     public function getEvent()
     {
@@ -316,7 +318,8 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
         }
 
         if (($result !== true) && $this->vars->sendupdates) {
-            Kronolith::sendITipNotifications($event, $GLOBALS['notification'], Kronolith::ITIP_REQUEST);
+            $type = $event->status == Kronolith::STATUS_CANCELLED ? Kronolith::ITIP_CANCEL : Kronolith::ITIP_REQUEST;
+            Kronolith::sendITipNotifications($event, $GLOBALS['notification'], $type);
         }
         Kronolith::notifyOfResourceRejection($event);
 
@@ -466,6 +469,7 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
                 $GLOBALS['notification']->push(_("You do not have permission to delete this event."), 'horde.warning');
                 return $result;
             }
+            $range = null;
             if ($event->recurs() && $this->vars->r != 'all') {
                 switch ($this->vars->r) {
                 case 'future':
@@ -486,6 +490,7 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
                         $event->recurrence->setRecurEnd($recurEnd);
                         $result = $this->_saveEvent($event, $event, $this->vars);
                     }
+                    $range = Kronolith::RANGE_THISANDFUTURE;
                     break;
                 case 'current':
                     // Deleting only the current instance.
@@ -504,7 +509,7 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
 
             if ($this->vars->sendupdates) {
                 Kronolith::sendITipNotifications(
-                    $event, $GLOBALS['notification'], Kronolith::ITIP_CANCEL, $instance);
+                    $event, $GLOBALS['notification'], Kronolith::ITIP_CANCEL, $instance, $range);
             }
             $result->deleted = true;
         } catch (Horde_Exception_NotFound $e) {
@@ -964,9 +969,13 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
                         $all_external = $GLOBALS['session']->get('kronolith', 'all_external_calendars');
                         $all_external[] = array('a' => 'tasks', 'n' => $tasklistId, 'd' => $tasklist->get('name'));
                         $GLOBALS['session']->set('kronolith', 'all_external_calendars', $all_external);
-                        $GLOBALS['display_external_calendars'][] = 'tasks/' . $tasklistId;
-                        $GLOBALS['prefs']->setValue('display_external_cals', serialize($GLOBALS['display_external_calendars']));
-                        $GLOBALS['all_external_calendars']['tasks/' . $tasklistId] = $wrapper;
+                        $display_external = $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_EXTERNAL_CALENDARS);
+                        $display_external[] = 'tasks/' . $tasklistId;
+                        $GLOBALS['calendar_manager']->set(Kronolith::DISPLAY_EXTERNAL_CALENDARS, $display_external);
+                        $GLOBALS['prefs']->setValue('display_external_cals', serialize($display_external));
+                        $all_external = $GLOBALS['calendar_manager']->get(Kronolith::ALL_EXTERNAL_CALENDARS);
+                        $all_external['tasks/' . $tasklistId] = $wrapper;
+                        $GLOBALS['calendar_manager']->set(Kronolith::ALL_EXTERNAL_CALENDARS, $all_external);
 
                         $result->saved = true;
                         $result->id = 'tasks/' . $tasklistId;
@@ -1193,10 +1202,11 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
     public function getCalendar()
     {
         $result = new stdClass;
-        if (!isset($GLOBALS['all_calendars'][$this->vars->cal]) && !$GLOBALS['conf']['share']['hidden']) {
+        $all_calendars = $GLOBALS['calendar_manager']->get(Kronolith::ALL_CALENDARS);
+        if (!isset($all_calendars[$this->vars->cal]) && !$GLOBALS['conf']['share']['hidden']) {
                 $GLOBALS['notification']->push(_("You are not allowed to view this calendar."), 'horde.error');
                 return $result;
-        } elseif (!isset($GLOBALS['all_calendars'][$this->vars->cal])) {
+        } elseif (!isset($all_calendars[$this->vars->cal])) {
             // Subscribing to a "hidden" share, check perms.
             $kronolith_shares = $GLOBALS['injector']->getInstance('Kronolith_Shares');
             $share = $kronolith_shares->getShare($this->vars->cal);
@@ -1206,7 +1216,7 @@ class Kronolith_Ajax_Application_Handler extends Horde_Core_Ajax_Application_Han
             }
             $calendar = new Kronolith_Calendar_Internal(array('share' => $share));
         } else {
-            $calendar = $GLOBALS['all_calendars'][$this->vars->cal];
+            $calendar = $all_calendars[$this->vars->cal];
         }
 
         $result->calendar = $calendar->toHash();
@@ -1382,6 +1392,63 @@ EOT;
         return new Horde_Core_Ajax_Response_Raw($js, 'text/javascript');
     }
 
+    public function toTimeslice()
+    {
+        $driver = $this->_getDriver($this->vars->cal);
+        $event = $driver->getEvent($this->vars->e);
+
+        try {
+            Kronolith::toTimeslice($event, $this->vars->t, $this->vars->c);
+        } catch (Kronolith_Exception $e) {
+            $GLOBALS['notification']->push(sprintf(_("Error saving timeslice: %s"), $e->getMessage()), 'horde.error');
+            return false;
+        }
+        $GLOBALS['notification']->push(_("Successfully saved timeslice."), 'horde.success');
+
+        return true;
+    }
+
+    /**
+     * Check reply status of any resources and report back. Used as a check
+     * before saving an event to give the user feedback.
+     *
+     * The following arguments are expected:
+     *   - r:  An array of resource identifiers.
+     *   - s:  The event start time to check.
+     *   - e:  The event end time to check.
+     *   - u:  The event uid, if not a new event.
+     *   - c:  The event's calendar.
+     */
+    public function checkResources()
+    {
+        if (empty($GLOBALS['conf']['resource']['driver'])) {
+            return array();
+        }
+
+        if ($this->vars->i) {
+            $event = $this->_getDriver($this->vars->c)->getEvent($this->vars->i);
+        } else {
+            $event = Kronolith::getDriver()->getEvent();
+        }
+        // Overrite start/end times since we may be checking before we edit
+        // an existing event with new times.
+        $event->start = new Horde_Date($this->vars->s);
+        $event->end = new Horde_Date($this->vars->e);
+        $event->start->setTimezone(date_default_timezone_get());
+        $event->end->setTimezone(date_default_timezone_get());
+        $results = array();
+        $r = $this->vars->r;
+        if (!is_array($r)) {
+            $r = array($r);
+        }
+        foreach ($r as $id) {
+            $resource = Kronolith::getDriver('Resource')->getResource($id);
+            $results[$id] = $resource->getResponse($event);
+        }
+
+        return $results;
+    }
+
     /**
      * Returns the driver object for a calendar.
      *
