@@ -5,7 +5,7 @@
  * This file defines Horde's core API interface. Other core Horde libraries
  * can interact with Kronolith through this API.
  *
- * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -15,7 +15,7 @@
 
 /* Determine the base directories. */
 if (!defined('KRONOLITH_BASE')) {
-    define('KRONOLITH_BASE', __DIR__ . '/..');
+    define('KRONOLITH_BASE', realpath(__DIR__ . '/..'));
 }
 
 if (!defined('HORDE_BASE')) {
@@ -24,7 +24,7 @@ if (!defined('HORDE_BASE')) {
     if (file_exists(KRONOLITH_BASE . '/config/horde.local.php')) {
         include KRONOLITH_BASE . '/config/horde.local.php';
     } else {
-        define('HORDE_BASE', KRONOLITH_BASE . '/..');
+        define('HORDE_BASE', realpath(KRONOLITH_BASE . '/..'));
     }
 }
 
@@ -32,6 +32,8 @@ if (!defined('HORDE_BASE')) {
  * Horde_Registry_Application::). */
 require_once HORDE_BASE . '/lib/core.php';
 
+use Sabre\CalDAV;
+
 class Kronolith_Application extends Horde_Registry_Application
 {
     /**
@@ -45,7 +47,7 @@ class Kronolith_Application extends Horde_Registry_Application
 
     /**
      */
-    public $version = 'H5 (4.1.4)';
+    public $version = 'H5 (4.2.1)';
 
     /**
      * Global variables defined:
@@ -67,6 +69,14 @@ class Kronolith_Application extends Horde_Registry_Application
         $GLOBALS['injector']->bindFactory('Kronolith_Geo', 'Kronolith_Factory_Geo', 'create');
         $GLOBALS['injector']->bindFactory('Kronolith_Shares', 'Kronolith_Factory_Shares', 'create');
 
+        /* Set the timezone variable, if available. */
+        $GLOBALS['registry']->setTimeZone();
+
+        /* Store the request timestamp if it's not already present. */
+        if (!isset($_SERVER['REQUEST_TIME'])) {
+            $_SERVER['REQUEST_TIME'] = time();
+        }
+
         if (!$GLOBALS['prefs']->getValue('dynamic_view')) {
             $this->features['dynamicView'] = false;
         }
@@ -77,17 +87,6 @@ class Kronolith_Application extends Horde_Registry_Application
         }
     }
 
-    protected function _authenticated()
-    {
-        /* Set the timezone variable, if available. */
-        $GLOBALS['registry']->setTimeZone();
-
-        /* Store the request timestamp if it's not already present. */
-        if (!isset($_SERVER['REQUEST_TIME'])) {
-            $_SERVER['REQUEST_TIME'] = time();
-        }
-    }
-
     /**
      */
     public function perms()
@@ -96,7 +95,10 @@ class Kronolith_Application extends Horde_Registry_Application
             'max_events' => array(
                 'title' => _("Maximum Number of Events"),
                 'type' => 'int'
-            )
+            ),
+            'resource_management' => array(
+                'title' => _("Resource Management"),
+                'type' => 'boolean')
         );
     }
 
@@ -222,7 +224,7 @@ class Kronolith_Application extends Horde_Registry_Application
         );
         foreach (Kronolith::listInternalCalendars() as $id => $calendar) {
             $row = array(
-                'selected' => in_array($id, $GLOBALS['display_calendars']),
+                'selected' => in_array($id, $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_CALENDARS)),
                 'url' => $url->copy()->add('toggle_calendar', $id),
                 'label' => Kronolith::getLabel($calendar),
                 'color' => Kronolith::backgroundColor($calendar),
@@ -239,7 +241,7 @@ class Kronolith_Application extends Horde_Registry_Application
         if ($GLOBALS['registry']->isAdmin()) {
             foreach ($GLOBALS['injector']->getInstance('Kronolith_Shares')->listSystemShares() as $id => $calendar) {
                 $row = array(
-                    'selected' => in_array($id, $GLOBALS['display_calendars']),
+                    'selected' => in_array($id, $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_CALENDARS)),
                     'url' => $url->copy()->add('toggle_calendar', $id),
                     'label' => $calendar->get('name'),
                     'color' => Kronolith::backgroundColor($calendar),
@@ -248,60 +250,61 @@ class Kronolith_Application extends Horde_Registry_Application
                 );
                 $sidebar->addRow($row, 'system');
             }
+        }
+        if (!empty($GLOBALS['conf']['resource']['driver']) &&
+            ($GLOBALS['registry']->isAdmin() || $GLOBALS['injector']->getInstance('Horde_Core_Perms')->hasAppPermission('resource_management'))) {
 
-            if (!empty($GLOBALS['conf']['resource']['driver'])) {
-                $sidebar->containers['groups'] = array(
-                    'header' => array(
-                        'id' => 'kronolith-toggle-groups',
-                        'label' => _("Resource Groups"),
-                        'collapsed' => true,
-                        'add' => array(
-                            'url' => Horde::url('resources/groups/create.php'),
-                            'label' => _("Create a new Resource Group"),
-                        ),
+            $sidebar->containers['groups'] = array(
+                'header' => array(
+                    'id' => 'kronolith-toggle-groups',
+                    'label' => _("Resource Groups"),
+                    'collapsed' => true,
+                    'add' => array(
+                        'url' => Horde::url('resources/groups/create.php'),
+                        'label' => _("Create a new Resource Group"),
                     ),
-                );
-                $editGroups = Horde::url('resources/groups/edit.php');
-                $sidebar->containers['resources'] = array(
-                    'header' => array(
-                        'id' => 'kronolith-toggle-resources',
-                        'label' => _("Resources"),
-                        'collapsed' => true,
-                        'add' => array(
-                            'url' => Horde::url('resources/create.php'),
-                            'label' => _("Create a new Resource"),
-                        ),
+                ),
+            );
+            $editGroups = Horde::url('resources/groups/edit.php');
+            $sidebar->containers['resources'] = array(
+                'header' => array(
+                    'id' => 'kronolith-toggle-resources',
+                    'label' => _("Resources"),
+                    'collapsed' => true,
+                    'add' => array(
+                        'url' => Horde::url('resources/create.php'),
+                        'label' => _("Create a new Resource"),
                     ),
-                );
-                $edit = Horde::url('resources/edit.php');
-                foreach (Kronolith::getDriver('Resource')->listResources() as $resource) {
-                    if ($resource->get('type') == Kronolith_Resource::TYPE_GROUP) {
-                        $row = array(
-                            'label' => $resource->get('name'),
-                            'color' => '#dddddd',
-                            'edit' => $editGroups->add('c', $resource->getId()),
-                            'type' => 'radiobox',
-                        );
-                        $sidebar->addRow($row, 'groups');
-                    } else {
-                        $calendar = new Kronolith_Calendar_Resource(array(
-                            'resource' => $resource
-                        ));
-                        $row = array(
-                            'selected' => in_array($resource->get('calendar'), $GLOBALS['display_resource_calendars']),
-                            'url' => $url->copy()->add('toggle_calendar', 'resource_' . $resource->get('calendar')),
-                            'label' => $calendar->name(),
-                            'color' => $calendar->background(),
-                            'edit' => $edit->add('c', $resource->getId()),
-                            'type' => 'checkbox',
-                        );
-                        $sidebar->addRow($row, 'resources');
-                    }
+                ),
+            );
+            $edit = Horde::url('resources/edit.php');
+            foreach (Kronolith::getDriver('Resource')->listResources() as $resource) {
+                if ($resource->get('type') == Kronolith_Resource::TYPE_GROUP) {
+                    $row = array(
+                        'label' => $resource->get('name'),
+                        'color' => '#dddddd',
+                        'edit' => $editGroups->add('c', $resource->getId()),
+                        'type' => 'radiobox',
+                    );
+                    $sidebar->addRow($row, 'groups');
+                } else {
+                    $calendar = new Kronolith_Calendar_Resource(array(
+                        'resource' => $resource
+                    ));
+                    $row = array(
+                        'selected' => in_array($resource->get('calendar'), $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_RESOURCE_CALENDARS)),
+                        'url' => $url->copy()->add('toggle_calendar', 'resource_' . $resource->get('calendar')),
+                        'label' => $calendar->name(),
+                        'color' => $calendar->background(),
+                        'edit' => $edit->add('c', $resource->getId()),
+                        'type' => 'checkbox',
+                    );
+                    $sidebar->addRow($row, 'resources');
                 }
             }
         }
 
-        foreach ($GLOBALS['all_external_calendars'] as $id => $calendar) {
+        foreach ($GLOBALS['calendar_manager']->get(Kronolith::ALL_EXTERNAL_CALENDARS) as $id => $calendar) {
             if (!$calendar->display()) {
                 continue;
             }
@@ -322,7 +325,7 @@ class Kronolith_Application extends Horde_Registry_Application
                 );
             }
             $row = array(
-                'selected' => in_array($id, $GLOBALS['display_external_calendars']),
+                'selected' => in_array($id, $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_EXTERNAL_CALENDARS)),
                 'url' => $url->copy()->add('toggle_calendar', 'external_' . $id),
                 'label' => $calendar->name(),
                 'color' => $calendar->background(),
@@ -343,9 +346,9 @@ class Kronolith_Application extends Horde_Registry_Application
             ),
         );
         $edit = Horde::url('calendars/remote_edit.php');
-        foreach ($GLOBALS['all_remote_calendars'] as $id => $calendar) {
+        foreach ($GLOBALS['calendar_manager']->get(Kronolith::ALL_REMOTE_CALENDARS) as $calendar) {
             $row = array(
-                'selected' => in_array($calendar->url(), $GLOBALS['display_remote_calendars']),
+                'selected' => in_array($calendar->url(), $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_REMOTE_CALENDARS)),
                 'url' => $url->copy()->add('toggle_calendar', 'remote_' . $calendar->url()),
                 'label' => $calendar->name(),
                 'color' => $calendar->background(),
@@ -363,9 +366,9 @@ class Kronolith_Application extends Horde_Registry_Application
                     'collapsed' => true,
                 ),
             );
-            foreach ($GLOBALS['all_holidays'] as $id => $calendar) {
+            foreach ($GLOBALS['calendar_manager']->get(Kronolith::ALL_HOLIDAYS) as $id => $calendar) {
                 $row = array(
-                    'selected' => in_array($id, $GLOBALS['display_holidays']),
+                    'selected' => in_array($id, $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_HOLIDAYS)),
                     'url' => $url->copy()->add('toggle_calendar', 'holiday_' . $id),
                     'label' => $calendar->name(),
                     'color' => $calendar->background(),
@@ -409,7 +412,7 @@ class Kronolith_Application extends Horde_Registry_Application
                 $kronolith_shares->removeShare($share);
             }
         } catch (Exception $e) {
-            Horde::logMessage($e, 'NOTICE');
+            Horde::log($e, 'NOTICE');
             $error = true;
         }
 
@@ -421,7 +424,7 @@ class Kronolith_Application extends Horde_Registry_Application
                 $share->removeUser($user);
             }
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e, 'NOTICE');
+            Horde::log($e, 'NOTICE');
             $error = true;
         }
 
@@ -485,7 +488,7 @@ class Kronolith_Application extends Horde_Registry_Application
         $time = new Horde_Date($time);
         $calendars = is_null($user)
             ? array_keys($kronolith_shares->listAllShares())
-            : $GLOBALS['display_calendars'];
+            : $GLOBALS['calendar_manager']->get(Kronolith::DISPLAY_CALENDARS);
         $alarms = Kronolith::listAlarms($time, $calendars, true);
         foreach ($alarms as $calendar => $cal_alarms) {
             if (!$cal_alarms) {
@@ -572,6 +575,11 @@ class Kronolith_Application extends Horde_Registry_Application
             foreach ($calendars as $calendar) {
                 list($type, $cal) = explode('_', $calendar, 2);
                 $kronolith_driver = Kronolith::getDriver($type, $cal);
+                $calendarObject = Kronolith::getCalendar($kronolith_driver);
+                if (!$calendarObject ||
+                    !$calendarObject->hasPermission(Horde_Perms::READ)) {
+                    throw new Horde_Exception_PermissionDenied();
+                }
                 $events[$calendar] = $kronolith_driver->listEvents(
                     $start,
                     $end,
@@ -607,7 +615,9 @@ class Kronolith_Application extends Horde_Registry_Application
 
                             if ($event->recurs()) {
                                 $row['recur_type'] = $event->recurrence->getRecurType();
-                                $row['recur_end_date'] = $event->recurrence->recurEnd->format('Y-m-d');
+                                if ($event->recurrence->hasRecurEnd()) {
+                                    $row['recur_end_date'] = $event->recurrence->recurEnd->format('Y-m-d');
+                                }
                                 $row['recur_interval'] = $event->recurrence->getRecurInterval();
                                 $row['recur_data'] = $event->recurrence->recurData;
                             }
@@ -617,25 +627,24 @@ class Kronolith_Application extends Horde_Registry_Application
                     }
                 }
 
-                $injector->getInstance('Horde_Core_Factory_Data')->create('Csv', array('cleanup' => array($this, 'cleanupData')))->exportFile(_("events.csv"), $data, true);
+                $injector->getInstance('Horde_Core_Factory_Data')
+                    ->create('Csv', array('cleanup' => array($this, 'cleanupData')))
+                    ->exportFile(_("events.csv"), $data, true);
                 exit;
 
             case Horde_Data::EXPORT_ICALENDAR:
-                $calNames = $calIds = array();
+                $calNames = array();
                 $iCal = new Horde_Icalendar();
 
                 foreach ($events as $calevents) {
                     foreach ($calevents as $dayevents) {
                         foreach ($dayevents as $event) {
-                            $calIds[$event->calendar] = true;
                             $calNames[Kronolith::getCalendar($event->getDriver())->name()] = true;
                             $iCal->addComponent($event->toiCalendar($iCal));
                         }
                     }
                 }
 
-                $kshares = $injector->getInstance('Kronolith_Shares');
-
                 $iCal->setAttribute('X-WR-CALNAME', implode(', ', array_keys($calNames)));
 
                 return array(
@@ -662,12 +671,9 @@ class Kronolith_Application extends Horde_Registry_Application
     public function davGetCollections($user)
     {
         $opts = array('perm' => Horde_Perms::SHOW);
-        if ($user != '-system-') {
-            $opts['attributes'] = $user;
-        }
         $shares = $GLOBALS['injector']
             ->getInstance('Kronolith_Shares')
-            ->listShares($GLOBALS['registry']->getAuth(), $opts);
+            ->listShares($user, $opts);
         $dav = $GLOBALS['injector']
             ->getInstance('Horde_Dav_Storage');
         $calendars = array();
@@ -675,6 +681,9 @@ class Kronolith_Application extends Horde_Registry_Application
             if ($user == '-system-' && $share->get('owner')) {
                 continue;
             }
+            $calendar = $GLOBALS['calendar_manager']
+                ->getEntry(Kronolith::ALL_CALENDARS, $id)
+                ->toHash();
             try {
                 $id = $dav->getExternalCollectionId($id, 'calendar');
             } catch (Horde_Dav_Exception $e) {
@@ -682,13 +691,18 @@ class Kronolith_Application extends Horde_Registry_Application
             $calendars[] = array(
                 'id' => $id,
                 'uri' => $id,
+                '{' . CalDAV\Plugin::NS_CALENDARSERVER . '}shared-url' =>
+                    $calendar['caldav'],
                 'principaluri' => 'principals/' . $user,
+                '{http://sabredav.org/ns}owner-principal' =>
+                    'principals/' . ($share->get('owner') ?: '-system-'),
                 '{DAV:}displayname' => Kronolith::getLabel($share),
-                '{urn:ietf:params:xml:ns:caldav}calendar-description' =>
+                '{' . CalDAV\Plugin::NS_CALDAV . '}calendar-description' =>
                     $share->get('desc'),
                 '{http://apple.com/ns/ical/}calendar-color' =>
-                    $share->get('color'),
-                '{urn:ietf:params:xml:ns:caldav}supported-calendar-component-set' => new Sabre\CalDAV\Property\SupportedCalendarComponentSet(array('VEVENT')),
+                    $share->get('color') . 'ff',
+                '{' . CalDAV\Plugin::NS_CALDAV . '}supported-calendar-component-set' => new CalDAV\Property\SupportedCalendarComponentSet(array('VEVENT')),
+                '{http://sabredav.org/ns}read-only' => !$share->hasPermission($user, Horde_Perms::EDIT),
             );
         }
         return $calendars;
@@ -703,7 +717,7 @@ class Kronolith_Application extends Horde_Registry_Application
 
         $internal = $dav->getInternalCollectionId($collection, 'calendar') ?: $collection;
         if (!Kronolith::hasPermission($internal, Horde_Perms::READ)) {
-            throw new Kronolith_Exception("Calendar does not exist or no permission to edit");
+            throw new Kronolith_Exception(_("Calendar does not exist or no permission to edit"));
         }
 
         $kronolith_driver = Kronolith::getDriver(null, $internal);
@@ -744,7 +758,7 @@ class Kronolith_Application extends Horde_Registry_Application
 
         $internal = $dav->getInternalCollectionId($collection, 'calendar') ?: $collection;
         if (!Kronolith::hasPermission($internal, Horde_Perms::READ)) {
-            throw new Kronolith_Exception("Calendar does not exist or no permission to edit");
+            throw new Kronolith_Exception(_("Calendar does not exist or no permission to edit"));
         }
 
         $kronolith_driver = Kronolith::getDriver(null, $internal);
@@ -790,7 +804,7 @@ class Kronolith_Application extends Horde_Registry_Application
 
         $internal = $dav->getInternalCollectionId($collection, 'calendar') ?: $collection;
         if (!Kronolith::hasPermission($internal, Horde_Perms::EDIT)) {
-            throw new Kronolith_Exception("Calendar does not exist or no permission to edit");
+            throw new Kronolith_Exception(_("Calendar does not exist or no permission to edit"));
         }
 
         $ical = new Horde_Icalendar();
@@ -806,7 +820,7 @@ class Kronolith_Application extends Horde_Registry_Application
             }
 
             $event = $kronolith_driver->getEvent();
-            $event->fromiCalendar($content);
+            $event->fromiCalendar($content, true);
 
             try {
                 try {
@@ -843,6 +857,15 @@ class Kronolith_Application extends Horde_Registry_Application
             if (!$existing_event) {
                 $dav->addObjectMap($id, $object, $internal);
             }
+
+            // Send iTip messages.
+            // Notifications will get lost, there is no way to return messages
+            // to clients.
+            Kronolith::sendITipNotifications(
+                $event,
+                new Horde_Notification_Handler(new Horde_Notification_Storage_Object()),
+                Kronolith::ITIP_REQUEST
+            );
         }
     }
 
@@ -854,7 +877,7 @@ class Kronolith_Application extends Horde_Registry_Application
 
         $internal = $dav->getInternalCollectionId($collection, 'calendar') ?: $collection;
         if (!Kronolith::hasPermission($internal, Horde_Perms::DELETE)) {
-            throw new Kronolith_Exception("Calendar does not exist or no permission to delete");
+            throw new Kronolith_Exception(_("Calendar does not exist or no permission to delete"));
         }
 
         try {
@@ -862,11 +885,23 @@ class Kronolith_Application extends Horde_Registry_Application
                 ?: preg_replace('/\.ics$/', '', $object);
         } catch (Horde_Dav_Exception $e) {
         }
-        Kronolith::getDriver(null, $internal)->deleteEvent($object);
+
+        $kronolith_driver = Kronolith::getDriver(null, $internal);
+        $event = $kronolith_driver->getEvent($object);
+        $kronolith_driver->deleteEvent($object);
 
         try {
             $dav->deleteExternalObjectId($object, $internal);
         } catch (Horde_Dav_Exception $e) {
         }
+
+        // Send iTip messages.
+        // Notifications will get lost, there is no way to return messages to
+        // clients.
+        Kronolith::sendITipNotifications(
+            $event,
+            new Horde_Notification_Handler(new Horde_Notification_Storage_Object()),
+            Kronolith::ITIP_CANCEL
+        );
     }
 }
