@@ -3,7 +3,7 @@
  * Kronolith_Driver defines an API for implementing storage backends for
  * Kronolith.
  *
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -383,7 +383,7 @@ class Kronolith_Driver
      */
     public function exists($uid, $calendar_id = null)
     {
-        throw new Kronolith_Exception('Not supported');
+        throw new BadMethodCallException('Not supported');
     }
 
     /**
@@ -407,7 +407,7 @@ class Kronolith_Driver
                 $history->log('kronolith:' . $event->calendar . ':' . $uid, array('action' => 'delete'), true);
                 $history->log('kronolith:' . $newCalendar . ':' . $uid, array('action' => 'add'), true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
     }
@@ -419,7 +419,7 @@ class Kronolith_Driver
      */
     protected function _move($eventId, $newCalendar)
     {
-        throw new Kronolith_Exception('Not supported');
+        throw new BadMethodCallException('Not supported');
     }
 
     /**
@@ -429,7 +429,7 @@ class Kronolith_Driver
      */
     public function delete($calendar)
     {
-        throw new Kronolith_Exception('Not supported');
+        throw new BadMethodCallException('Not supported');
     }
 
     /**
@@ -456,7 +456,7 @@ class Kronolith_Driver
             try {
                 $GLOBALS['injector']->getInstance('Horde_History')->log('kronolith:' . $this->calendar . ':' . $event->uid, array('action' => 'delete'), true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
 
@@ -478,7 +478,7 @@ class Kronolith_Driver
 
         /* Remove any tags */
         $tagger = Kronolith::getTagger();
-        $tagger->replaceTags($event->uid, array(), $event->creator, 'event');
+        $tagger->replaceTags($event->uid, array(), $event->creator, Kronolith_Tagger::TYPE_EVENT);
 
         /* Remove any geolocation data. */
         try {
@@ -504,7 +504,7 @@ class Kronolith_Driver
             try {
                 $GLOBALS['injector']->getInstance('Horde_History')->log('kronolith:' . $this->calendar . ':' . $event->baseid, array('action' => 'modify'), true);
             } catch (Exception $e) {
-                Horde::logMessage($e, 'ERR');
+                Horde::log($e, 'ERR');
             }
         }
     }
@@ -545,7 +545,7 @@ class Kronolith_Driver
      */
     public function filterEventsByCalendar($uids, $calendar)
     {
-        throw new Kronolith_Exception('Not supported');
+        throw new BadMethodCallException('Not supported');
     }
 
     /**
@@ -557,7 +557,7 @@ class Kronolith_Driver
      */
     public function removeUserData($user)
     {
-        throw new Kronolith_Exception('Deprecated.');
+        throw new BadMethodCallException('Deprecated.');
     }
 
     /**
@@ -567,7 +567,7 @@ class Kronolith_Driver
      */
     protected function _updateTags(Kronolith_Event $event)
     {
-        Kronolith::getTagger()->replaceTags($event->uid, $event->tags, $event->creator, 'event');
+        Kronolith::getTagger()->replaceTags($event->uid, $event->tags, $event->creator, Kronolith_Tagger::TYPE_EVENT);
 
         // Resources don't currently have owners, so can't tag as owner.
         if ($event->calendarType == 'resource') {
@@ -580,7 +580,7 @@ class Kronolith_Driver
         } catch (Horde_Share_Exception $e) {
             throw new Kronolith_Exception($e);
         }
-        Kronolith::getTagger()->tag($event->uid, $event->tags, $cal->get('owner'), 'event');
+        Kronolith::getTagger()->tag($event->uid, $event->tags, $cal->get('owner'), Kronolith_Tagger::TYPE_EVENT);
     }
 
     /**
@@ -591,7 +591,7 @@ class Kronolith_Driver
     protected function _addTags(Kronolith_Event $event)
     {
         $tagger = Kronolith::getTagger();
-        $tagger->tag($event->uid, $event->tags, $event->creator, 'event');
+        $tagger->tag($event->uid, $event->tags, $event->creator, Kronolith_Tagger::TYPE_EVENT);
 
         // Resources don't currently have owners, so can't tag as owner.
         if ($event->calendarType == 'resource') {
@@ -602,7 +602,7 @@ class Kronolith_Driver
         try {
             $cal = $GLOBALS['injector']->getInstance('Kronolith_Shares')->getShare($event->calendar);
         } catch (Horde_Share_Exception $e) {
-            Horde::logMessage($e->getMessage(), 'ERR');
+            Horde::log($e->getMessage(), 'ERR');
             throw new Kronolith_Exception($e);
         }
 
