@@ -3,7 +3,7 @@
  * The Kronolith_Driver_Resource class implements the Kronolith_Driver API for
  * storing resource calendars in a SQL backend.
  *
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.horde.org/licenses/gpl.
@@ -23,6 +23,13 @@ class Kronolith_Driver_Resource_Sql extends Kronolith_Driver
      */
     protected $_driver;
 
+    /**
+     * Column information as Horde_Db_Adapter_Base_Column objects.
+     *
+     * @var array
+     */
+    protected $_columns = array();
+
     /**
      * The class name of the event object to instantiate.
      *
@@ -51,6 +58,7 @@ class Kronolith_Driver_Resource_Sql extends Kronolith_Driver
         ), $this->_params);
 
         $this->_driver = Kronolith::getDriver();
+        $this->_columns = $this->_db->columns($this->_params['table']);
     }
 
     /**
@@ -217,10 +225,14 @@ class Kronolith_Driver_Resource_Sql extends Kronolith_Driver
      * @param Kronolith_Resource_Base $resource
      *
      * @return Kronolith_Resource object
-     * @throws Kronolith_Exception
+     * @throws Kronolith_Exception, Horde_Exception_PermissionDenied
      */
     public function save(Kronolith_Resource_Base $resource)
     {
+        if (!$GLOBALS['registry']->isAdmin() &&
+            !$GLOBALS['injector']->getInstance('Horde_Core_Perms')->hasAppPermission('resource_management')) {
+            throw new Horde_Exception_PermissionDenied();
+        }
         if ($resource->getId()) {
             $query = 'UPDATE ' . $this->_params['table'] . ' SET resource_name = ?, '
                 . 'resource_calendar = ? , resource_description = ?, '
@@ -271,10 +283,15 @@ class Kronolith_Driver_Resource_Sql extends Kronolith_Driver
      *
      * @param Kronolith_Resource_Base $resource  The kronolith resource to remove
      *
-     * @throws Kronolith_Exception
+     * @throws Kronolith_Exception, Horde_Exception_PermissionDenied
      */
     public function delete($resource)
     {
+        if (!$GLOBALS['registry']->isAdmin() &&
+            !$GLOBALS['injector']->getInstance('Horde_Core_Perms')->hasAppPermission('resource_management')) {
+            throw new Horde_Exception_PermissionDenied();
+        }
+
         if (!$resource->getId()) {
             throw new Kronolith_Exception(_("Resource not valid."));
         }
@@ -497,11 +514,21 @@ class Kronolith_Driver_Resource_Sql extends Kronolith_Driver
     protected function _fromDriver(array $params)
     {
         $return = array();
+
         foreach ($params as $field => $value) {
-            if ($field == 'resource_name' || $field == 'resource_description') {
+            switch ($field) {
+            case 'resource_description':
+                $value = $this->_columns['resource_description']
+                    ->binaryToString($value);
+                // Fall through.
+            case 'resource_name':
                 $value = $this->convertFromDriver($value);
-            } elseif ($field == 'resource_members') {
+                break;
+            case 'resource_members':
+                $value = $this->_columns['resource_members']
+                    ->binaryToString($value);
                 $value = @unserialize($value);
+                break;
             }
 
             $return[str_replace('resource_', '', $field)] = $value;
@@ -534,5 +561,4 @@ class Kronolith_Driver_Resource_Sql extends Kronolith_Driver
     {
         // noop
     }
-
 }
